@@ -22,7 +22,7 @@ def generate_inputs_and_wrap_model(config_path,
     For example, the MMDet models' forward function has a parameter
     ``return_loss:bool``. As we want to set it as False while export API
     supports neither bool type or kwargs. So we have to replace the forward
-    like: ``model.forward = partial(model.forward, return_loss=False)``
+    method like ``model.forward = partial(model.forward, return_loss=False)``.
 
     Args:
         config_path (str): the OpenMMLab config for the model we want to
@@ -35,9 +35,9 @@ def generate_inputs_and_wrap_model(config_path,
             as there is no legal bbox.
 
     Returns:
-        tuple: (model, tensor_data) wrapped model which can be called by \
-        model(*tensor_data) and a list of inputs which are used to execute \
-            the model while exporting.
+        tuple: (model, tensor_data) wrapped model which can be called by
+            ``model(*tensor_data)`` and a list of inputs which are used to
+            execute the model while exporting.
     """
 
     model = build_model_from_cfg(
@@ -156,6 +156,7 @@ def preprocess_example_input(input_config):
         'scale_factor': np.ones(4, dtype=np.float32),
         'flip': False,
         'show_img': show_img,
+        'flip_direction': None
     }
 
     return one_img, one_meta
