@@ -18,7 +18,8 @@ class PointRend(TwoStageDetector):
                  train_cfg,
                  test_cfg,
                  neck=None,
-                 pretrained=None):
+                 pretrained=None,
+                 init_cfg=None):
         super(PointRend, self).__init__(
             backbone=backbone,
             neck=neck,
@@ -26,4 +27,5 @@ class PointRend(TwoStageDetector):
             roi_head=roi_head,
             train_cfg=train_cfg,
             test_cfg=test_cfg,
-            pretrained=pretrained)
+            pretrained=pretrained,
+            init_cfg=init_cfg)
